Package: theorytools
Type: Package
Title: FAIR Theory Construction
Version: 0.1.3
Authors@R: c(
    person(
      given = c("Caspar", "J."),
      family = "Van Lissa",
      email = "c.j.vanlissa@tilburguniversity.edu",
      role = c("aut", "cre"),
      comment = c(ORCID = "0000-0002-0808-5024")
    )
    )
Description: An integrated suite of tools for creating, maintaining, and reusing
    FAIR (Findable, Accessible, Interoperable, Reusable) theories.
    Designed to support transparent and collaborative theory development,
    the package enables users to formalize theories,
    track changes with version control,
    assess pre-empirical coherence,
    and derive testable hypotheses.
    Aligning with open science principles and workflows,
    'theorytools' facilitates the systematic improvement of theoretical frameworks
    and enhances their discoverability and usability.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://cjvanlissa.github.io/theorytools/
RoxygenNote: 7.3.3
Imports: worcs (>= 0.1.19), gert, gh, cli, jsonlite, curl, knitr,
        dagitty, yaml, tidySEM (>= 0.2.9), methods, Deriv, stringdist
Suggests: rmarkdown, testthat (>= 3.0.0), pkgdown, webexercises, withr,
        usethis, fs, ggplot2, igraph, bookdown, reticulate
SystemRequirements: Python (>= 3.8)
VignetteBuilder: knitr
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
Config/Needs/website: rmarkdown
NeedsCompilation: no
Packaged: 2026-02-07 08:59:04 UTC; cjvanlissa
Author: Caspar J. Van Lissa [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0808-5024>)
Maintainer: Caspar J. Van Lissa <c.j.vanlissa@tilburguniversity.edu>
Repository: CRAN
Date/Publication: 2026-02-07 10:10:02 UTC
Built: R 4.6.0; ; 2026-04-25 14:06:19 UTC; unix
