# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,code_list)
S3method(code,character)
S3method(code,code_list)
S3method(most_similar,code_list)
S3method(most_similar,matrix)
S3method(print,code_list)
S3method(print,similarity_vector)
export(add_level)
export(add_readme_fair_theory)
export(add_significance)
export(add_theory_file)
export(add_to_pkgdown)
export(add_zenodo_json)
export(add_zenodo_json_theory)
export(code)
export(create_fair_theory)
export(derive_formula)
export(download_huggingface)
export(download_theory)
export(filter_conditional_independencies)
export(get_embeddings)
export(prune_dag)
export(select_controls)
export(similarity_cosine)
export(similarity_stringdist)
export(simulate_data)
export(use_webex_vignette)
importFrom(Deriv,Simplify)
importFrom(curl,curl_download)
importFrom(curl,curl_fetch_memory)
importFrom(dagitty,adjustmentSets)
importFrom(dagitty,dagitty)
importFrom(dagitty,exogenousVariables)
importFrom(gert,git_clone)
importFrom(gert,git_init)
importFrom(gert,git_remote_ls)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,write_json)
importFrom(knitr,is_html_output)
importFrom(methods,formalArgs)
importFrom(methods,hasArg)
importFrom(stats,as.formula)
importFrom(stats,formula)
importFrom(stats,model.frame)
importFrom(stats,na.omit)
importFrom(stats,reformulate)
importFrom(stats,runif)
importFrom(stats,terms)
importFrom(stringdist,stringdistmatrix)
importFrom(tidySEM,get_edges)
importFrom(tidySEM,get_nodes)
importFrom(tools,md5sum)
importFrom(utils,getFromNamespace)
importFrom(worcs,add_license_file)
importFrom(worcs,git_update)
importFrom(yaml,read_yaml)
importFrom(yaml,write_yaml)
